@@ -6,14 +6,13 @@
 #include <libgnomedb/libgnomedb.h>
 #include <libgnomedb-extra/gnome-db-sql-console.h>
 
-extern GdaDict *demo_dict;
+extern GdaConnection *demo_cnc;
 static GtkWidget *window = NULL;
 
 GtkWidget *
 do_console (GtkWidget *do_widget)
 {  
 	if (!window) {
-                GdaQuery *query;
 		GtkWidget *vbox;
 		GtkWidget *label;
 		GtkWidget *console;
@@ -38,7 +37,7 @@ do_console (GtkWidget *do_widget)
 		gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
 		
 		/* Create the demo widget */
-		console = gnome_db_sql_console_new (demo_dict, 
+		console = gnome_db_sql_console_new (demo_cnc, 
 						    "SQL Console (type \\? + ENTER for help)\n");
 		gtk_widget_set_size_request (console, 480, 320);
 
