@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1998 - 2007 The GNOME Foundation.
+ * Copyright (C) 1998 - 2008 The GNOME Foundation.
  *
  * AUTHORS:
  *	Michael Lausch <michael@lausch.at>
@@ -246,7 +246,11 @@ gnome_db_error_finalize (GObject *object)
 	g_return_if_fail (GNOME_DB_IS_ERROR (error_widget));
 
 	/* free memory */
-	gda_connection_event_list_free (error_widget->priv->current_list);
+	if (error_widget->priv->current_list) {
+		g_list_foreach (error_widget->priv->current_list, (GFunc) g_object_unref, NULL);
+		g_list_free (error_widget->priv->current_list);
+		error_widget->priv->current_list = NULL;
+	}
 	g_free (error_widget->priv);
 	error_widget->priv = NULL;
 
@@ -285,8 +289,11 @@ gnome_db_error_clear (GnomeDbError *error_widget)
 	gnome_db_text_clear (GTK_TEXT_VIEW (error_widget->priv->error_description));
 	gtk_entry_set_text (GTK_ENTRY (error_widget->priv->error_source), "");
 
-	gda_connection_event_list_free (error_widget->priv->current_list);
-	error_widget->priv->current_list = NULL;
+	if (error_widget->priv->current_list) {
+		g_list_foreach (error_widget->priv->current_list, (GFunc) g_object_unref, NULL);
+		g_list_free (error_widget->priv->current_list);
+		error_widget->priv->current_list = NULL;
+	}
 	error_widget->priv->nerrors = 0;
 	error_widget->priv->current_pos = 0;
 }
@@ -302,15 +309,14 @@ gnome_db_error_clear (GnomeDbError *error_widget)
 void
 gnome_db_error_show (GnomeDbError *error_widget, GList *errors)
 {
-	GList *new_list;
-
 	g_return_if_fail (GNOME_DB_IS_ERROR (error_widget));
 
 	gnome_db_error_clear (error_widget);
-	new_list = gda_connection_event_list_copy (errors);
-	error_widget->priv->nerrors = g_list_length (new_list);
 
-	error_widget->priv->current_list = new_list;
+	error_widget->priv->current_list = g_list_copy (errors);
+	g_list_foreach (error_widget->priv->current_list, (GFunc) g_object_ref, NULL);
+	error_widget->priv->nerrors = g_list_length (error_widget->priv->current_list);
+
 	display_current_error (error_widget);
 }
 
