@@ -312,7 +312,7 @@ gnome_db_data_cell_renderer_combo_set_property (GObject *object,
 							
 							value = gda_data_model_get_value_at (datacell->priv->source->data_model,
 											     datacell->priv->source->shown_cols_index [i],
-											     row);
+											     row, NULL);
 							dsplay_values = g_list_append (dsplay_values, (GValue *) value);
 						}
 						str = render_text_to_display_from_values (dsplay_values);
@@ -406,7 +406,7 @@ render_text_to_display_from_values (GList *values)
 			g_free (str);
 		}
 		else
-			g_string_append (string, " ");
+			g_string_append (string, " ? ");
 
 		list = g_list_next (list);
 	}
