@@ -2,17 +2,17 @@
  *
  * Copyright (C) 2006 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Library General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -169,28 +169,23 @@ data_widget_destroyed_cb (GnomeDbDataWidget *wid, GnomeDbDataWidgetInfo *info)
 
 
 static void
-proxy_destroyed_cb (GdaDataProxy *proxy, GnomeDbDataWidgetInfo *info)
+release_proxy (GnomeDbDataWidgetInfo *info)
 {
-	g_assert (proxy == info->priv->proxy);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (proxy),
-					      G_CALLBACK (proxy_destroyed_cb), info);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (proxy),
+	g_signal_handlers_disconnect_by_func (G_OBJECT (info->priv->proxy),
 					      G_CALLBACK (proxy_changed_cb), info);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (proxy),
+	g_signal_handlers_disconnect_by_func (G_OBJECT (info->priv->proxy),
 					      G_CALLBACK (proxy_sample_changed_cb), info);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (proxy),
+	g_signal_handlers_disconnect_by_func (G_OBJECT (info->priv->proxy),
 					      G_CALLBACK (proxy_row_changed_cb), info);
+	g_object_unref (info->priv->proxy);
 	info->priv->proxy = NULL;
 }
 
 static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbDataWidgetInfo *info);
 static void
-iter_destroyed_cb (GdaDataModelIter *iter, GnomeDbDataWidgetInfo *info)
+release_iter (GnomeDbDataWidgetInfo *info)
 {
-	g_assert (iter == info->priv->iter);
-	g_signal_handlers_disconnect_by_func (iter,
-					      G_CALLBACK (iter_destroyed_cb), info);
-	g_signal_handlers_disconnect_by_func (iter,
+	g_signal_handlers_disconnect_by_func (info->priv->iter,
 					      G_CALLBACK (iter_row_changed_cb), info);
 	info->priv->iter = NULL;
 }
@@ -212,9 +207,9 @@ gnome_db_data_widget_info_dispose (GObject *object)
 
 	if (info->priv) {
 		if (info->priv->proxy)
-			proxy_destroyed_cb (info->priv->proxy, info);
+			release_proxy (info);
 		if (info->priv->iter)
-			iter_destroyed_cb (info->priv->iter, info);
+			release_iter (info);
 		if (info->priv->data_widget)
 			data_widget_destroyed_cb (info->priv->data_widget, info);
 
@@ -242,11 +237,11 @@ gnome_db_data_widget_info_set_property (GObject *object,
 			if (info->priv->data_widget)
 				data_widget_destroyed_cb (info->priv->data_widget, info);
 			if (info->priv->iter) 
-				iter_destroyed_cb (info->priv->iter, info);
+				release_iter (info);
 			if (info->priv->proxy)
-				proxy_destroyed_cb (info->priv->proxy, info);
+				release_proxy (info);
 
-			info->priv->data_widget = GNOME_DB_DATA_WIDGET(g_value_get_object (value));
+			info->priv->data_widget = GNOME_DB_DATA_WIDGET (g_value_get_object (value));
 			if (info->priv->data_widget) {
 				GdaDataProxy *proxy;
 				GdaDataModelIter *iter;
@@ -264,8 +259,7 @@ gnome_db_data_widget_info_set_property (GObject *object,
 				proxy = gnome_db_data_widget_get_proxy (info->priv->data_widget);
 				if (proxy) {
 					info->priv->proxy = proxy;
-					gda_object_connect_destroy (proxy,
-								    G_CALLBACK (proxy_destroyed_cb), info);
+					g_object_ref (info->priv->proxy);
 					g_signal_connect (G_OBJECT (proxy), "changed",
 							  G_CALLBACK (proxy_changed_cb), info);
 					g_signal_connect (G_OBJECT (proxy), "sample_changed",
@@ -280,8 +274,7 @@ gnome_db_data_widget_info_set_property (GObject *object,
 										      (info->priv->data_widget));
 					info->priv->iter = iter;
 					if (iter) {
-						gda_object_connect_destroy (iter,
-									    G_CALLBACK (iter_destroyed_cb), info);
+						g_object_ref (G_OBJECT (iter));
 						g_signal_connect (iter, "row_changed",
 								  G_CALLBACK (iter_row_changed_cb), info);
 					}
@@ -391,6 +384,7 @@ static const gchar *ui_row_move =
 	"    <toolitem action='ActionPrevRecord'/>"
 	"    <toolitem action='ActionNextRecord'/>"
 	"    <toolitem action='ActionLastRecord'/>"
+	"    <toolitem action='ActionFilter'/>"
 	"  </toolbar>"
 	"</ui>";
 static const gchar *ui_chunck_change =
@@ -400,6 +394,7 @@ static const gchar *ui_chunck_change =
 	"    <toolitem action='ActionPrevChunck'/>"
 	"    <toolitem action='ActionNextChunck'/>"
 	"    <toolitem action='ActionLastChunck'/>"
+	"    <toolitem action='ActionFilter'/>"
 	"  </toolbar>"
 	"</ui>";
 
@@ -476,7 +471,7 @@ row_spin_changed_cb (GtkSpinButton *spin, GnomeDbDataWidgetInfo *info)
 	    (value <= gda_data_model_get_n_rows (GDA_DATA_MODEL (info->priv->proxy))))
 		row = value - 1;
 
-	gda_data_model_iter_set_at_row (gnome_db_data_widget_get_current_data (info->priv->data_widget), row);
+	gda_data_model_iter_move_at_row (gnome_db_data_widget_get_current_data (info->priv->data_widget), row);
 }
 
 #define BLOCK_SPIN (g_signal_handlers_block_by_func (G_OBJECT (info->priv->row_spin), \
@@ -487,7 +482,7 @@ static void
 modif_buttons_update (GnomeDbDataWidgetInfo *info)
 {
 	GdaDataModelIter *model_iter;
-	gboolean wrows;
+	gboolean wrows, filtered_proxy = FALSE;
 	gint has_selection;	
 	gint row;
 
@@ -498,12 +493,19 @@ modif_buttons_update (GnomeDbDataWidgetInfo *info)
 	GtkAction *action;
 	gint sample_first_row = 0, sample_last_row = 0, sample_size = 0;
 	GnomeDbDataWidgetInfoFlag flags = 0;
-	
+
 	model_iter = gnome_db_data_widget_get_current_data (info->priv->data_widget);
 	if (info->priv->proxy) {
+		filtered_proxy = gda_data_proxy_get_filter_expr (info->priv->proxy) ? TRUE : FALSE;
 		proxy_rows = gda_data_model_get_n_rows (GDA_DATA_MODEL (info->priv->proxy));
-		proxied_rows = gda_data_proxy_get_proxied_model_n_rows (info->priv->proxy);
-		all_rows = proxied_rows + gda_data_proxy_get_n_new_rows (info->priv->proxy);
+		if (filtered_proxy) {
+			proxied_rows = gda_data_proxy_get_filtered_n_rows (info->priv->proxy);
+			all_rows = proxied_rows;
+		}
+		else {
+			proxied_rows = gda_data_proxy_get_proxied_model_n_rows (info->priv->proxy);
+			all_rows = proxied_rows + gda_data_proxy_get_n_new_rows (info->priv->proxy);
+		}
 
 		/* samples don't take into account the proxy's inserted rows */
 		sample_first_row = gda_data_proxy_get_sample_start (info->priv->proxy);
@@ -517,7 +519,7 @@ modif_buttons_update (GnomeDbDataWidgetInfo *info)
 	wrows = (proxy_rows <= 0) ? FALSE : TRUE;
 	row = model_iter ? gda_data_model_iter_get_row (model_iter) : 0;
 	if (info->priv->flags & GNOME_DB_DATA_WIDGET_INFO_CURRENT_ROW) {
-		if (!wrows) {
+		if (proxy_rows < 0) {
 			if (info->priv->flags & GNOME_DB_DATA_WIDGET_INFO_ROW_MOVE_BUTTONS) {
 				BLOCK_SPIN;
 				gtk_spin_button_set_range (GTK_SPIN_BUTTON (info->priv->row_spin), 0, 1);
@@ -536,8 +538,12 @@ modif_buttons_update (GnomeDbDataWidgetInfo *info)
 			if (info->priv->flags & GNOME_DB_DATA_WIDGET_INFO_ROW_MOVE_BUTTONS) {
 				if (total <= 0)
 					str = g_strdup (" / 0");
-				else
-					str = g_strdup_printf (" / %d", proxy_rows);
+				else {
+					if (filtered_proxy)
+						str = g_strdup_printf (" / (%d)", proxy_rows);
+					else
+						str = g_strdup_printf (" / %d", proxy_rows);
+				}
 				BLOCK_SPIN;
 				gtk_spin_button_set_range (GTK_SPIN_BUTTON (info->priv->row_spin), 1, proxy_rows);
 				if (row >= 0)
@@ -551,11 +557,17 @@ modif_buttons_update (GnomeDbDataWidgetInfo *info)
 				else {
 					if (all_rows < 0)
 						str = g_strdup_printf ("%d - %d /?", sample_first_row + 1, total);
-					else
-						str = g_strdup_printf ("%d - %d / %d", 
-								       sample_first_row + 1, total, all_rows);
+					else {
+						if (filtered_proxy) 
+							str = g_strdup_printf ("%d - %d / (%d)", 
+									       sample_first_row + 1, total, all_rows);
+						else
+							str = g_strdup_printf ("%d - %d / %d", 
+									       sample_first_row + 1, total, all_rows);
+					}
 				}
 			}
+
 			gtk_label_set_text (GTK_LABEL (info->priv->current_sample), str);
 			g_free (str);
 		}
@@ -681,6 +693,12 @@ modif_buttons_update (GnomeDbDataWidgetInfo *info)
 		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionLastChunck");
 		g_object_set (G_OBJECT (action), "sensitive", wrows && (sample_last_row < proxied_rows - 1), NULL);
 	}
+
+	/* filter */
+	if (info->priv->flags & GNOME_DB_DATA_WIDGET_INFO_NO_FILTER) {
+		action = gtk_ui_manager_get_action (info->priv->uimanager, "/ToolBar/ActionFilter");
+		g_object_set (G_OBJECT (action), "visible", FALSE, NULL);
+	}
 	
 	if (info->priv->uimanager)
 		gtk_ui_manager_ensure_update (info->priv->uimanager); 
