@@ -332,11 +332,11 @@ static void
 dsn_entry_changed_cb (GnomeDbDsnSelector *sel, GnomeDbLogin *login)
 {
 	gchar *dsn;
-	GdaDataSourceInfo *info = NULL;
+	GdaDsnInfo *info = NULL;
         
         dsn = gnome_db_dsn_selector_get_dsn (sel);
         
-	info = gda_config_get_dsn (dsn);
+	info = gda_config_get_dsn_info (dsn);
 
 	if (info)
 		TO_IMPLEMENT;
@@ -388,10 +388,10 @@ gnome_db_login_get_dsn (GnomeDbLogin *login)
 void
 gnome_db_login_set_dsn (GnomeDbLogin *login, const gchar *dsn)
 {
-	GdaDataSourceInfo *src;
+	GdaDsnInfo *src;
 	g_return_if_fail (GNOME_DB_IS_LOGIN (login));
 
-	src = gda_config_get_dsn (dsn);
+	src = gda_config_get_dsn_info (dsn);
 	if (!src)
 		g_warning (_("Datasource '%s' is not declared"), dsn);
 	else {
