@@ -1,18 +1,18 @@
 /* gnome-db-raw-form.c
  *
- * Copyright (C) 2002 - 2007 Vivien Malerba
+ * Copyright (C) 2002 - 2008 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Library General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  * USA
@@ -44,7 +44,7 @@ static void gnome_db_raw_form_get_property (GObject *object,
 
 static void gnome_db_raw_form_initialize (GnomeDbRawForm *form, GtkWidget *layout, GHashTable *box_widgets);
 
-static gboolean iter_row_to_change_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form);
+static GError *iter_validate_set_cb (GdaDataModelIter *iter, GnomeDbRawForm *form);
 static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form);
 static void proxy_changed_cb (GdaDataProxy *proxy, GnomeDbRawForm *form);
 static void proxy_reset_cb (GdaDataProxy *proxy, GnomeDbRawForm *form);
@@ -186,10 +186,10 @@ static void
 action_new_activated_cb (GtkAction *action, GnomeDbRawForm *wid)
 {
 	/* make the first entry grab the focus */
-	if (wid->priv->iter && GDA_PARAMETER_LIST (wid->priv->iter)->parameters)
+	if (wid->priv->iter && GDA_SET (wid->priv->iter)->holders)
 		gnome_db_basic_form_entry_grab_focus (GNOME_DB_BASIC_FORM (wid), 
-						      (GdaParameter *)
-						      GDA_PARAMETER_LIST (wid->priv->iter)->parameters->data);
+						      (GdaHolder *)
+						      GDA_SET (wid->priv->iter)->holders->data);
 }
 
 static void
@@ -302,7 +302,7 @@ gnome_db_raw_form_dispose (GObject *object)
 			g_signal_handlers_disconnect_by_func (form->priv->iter,
 							      G_CALLBACK (iter_row_changed_cb), form);
 			g_signal_handlers_disconnect_by_func (form->priv->iter,
-							      G_CALLBACK (iter_row_to_change_cb), form);
+							      G_CALLBACK (iter_validate_set_cb), form);
 			g_object_unref (form->priv->iter);
 			form->priv->iter = NULL;
 		}
@@ -353,7 +353,7 @@ gnome_db_raw_form_set_property (GObject *object,
 				g_signal_handlers_disconnect_by_func (form->priv->iter,
 								      G_CALLBACK (iter_row_changed_cb), form);
 				g_signal_handlers_disconnect_by_func (form->priv->iter,
-								      G_CALLBACK (iter_row_to_change_cb), form);
+								      G_CALLBACK (iter_validate_set_cb), form);
 
 				g_object_unref (G_OBJECT (form->priv->iter));
 				form->priv->iter = NULL;
@@ -381,10 +381,10 @@ gnome_db_raw_form_set_property (GObject *object,
 				form->priv->model = gda_data_proxy_get_proxied_model (form->priv->proxy);
 
 				form->priv->iter = gda_data_model_create_iter ((GdaDataModel *) form->priv->proxy);
-				gda_data_model_iter_set_at_row (form->priv->iter, 0);
+				gda_data_model_iter_move_at_row (form->priv->iter, 0);
 
-				g_signal_connect (form->priv->iter, "row_to_change",
-						  G_CALLBACK (iter_row_to_change_cb), form);
+				g_signal_connect (form->priv->iter, "validate-set",
+						  G_CALLBACK (iter_validate_set_cb), form);
 				g_signal_connect (form->priv->iter, "row_changed",
 						  G_CALLBACK (iter_row_changed_cb), form);
 
@@ -434,27 +434,30 @@ gnome_db_raw_form_get_property (GObject *object,
         }	
 }
 
-static gboolean
-iter_row_to_change_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form)
+static GError *
+iter_validate_set_cb (GdaDataModelIter *iter, GnomeDbRawForm *form)
 {
-	gboolean allow_change = TRUE;
-	if (row >= 0) {
-		if (form->priv->write_mode >= GNOME_DB_DATA_WIDGET_WRITE_ON_ROW_CHANGE) {
-			/* write back the current row */
-			if (gda_data_proxy_row_has_changed (form->priv->proxy, row)) {
-				GError *error = NULL;
-				if (!gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error)) {
-					allow_change = gnome_db_utility_display_error_with_keep_or_discard_choice ((GnomeDbDataWidget *) form, 
-													  error);
-					if (allow_change)
-						gda_data_proxy_cancel_row_changes (form->priv->proxy, row, -1);
-					g_error_free (error);
-				}
+	GError *error = NULL;
+	gint row = gda_data_model_iter_get_row (iter);
+	
+	if (row < 0)
+		return NULL;
+
+	if ((form->priv->write_mode >= GNOME_DB_DATA_WIDGET_WRITE_ON_ROW_CHANGE) &&
+		/* write back the current row */
+	    gda_data_proxy_row_has_changed (form->priv->proxy, row) &&
+	    !gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error)) {
+		if (gnome_db_utility_display_error_with_keep_or_discard_choice ((GnomeDbDataWidget *) form, 
+										error)) {
+			gda_data_proxy_cancel_row_changes (form->priv->proxy, row, -1);
+			if (error) {
+				g_error_free (error);
+				error = NULL;
 			}
 		}
 	}
 	
-	return allow_change;
+	return error;
 }
 
 static void
@@ -466,14 +469,13 @@ iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form)
 	if (row >= 0) {
 		GSList *params;
 		guint attributes;
-		GdaParameter *param;
+		GdaHolder *param;
+		gint i;
 
-		params = ((GdaParameterList *) iter)->parameters;
-		while (params) {
+		for (i = 0, params = ((GdaSet *) iter)->holders; params; i++, params = params->next) {
 			gboolean explicit_hide = FALSE;
-			param = (GdaParameter *) params->data;
-			attributes = gda_data_proxy_get_value_attributes (form->priv->proxy, row, 
-							       gda_data_model_iter_get_column_for_param (iter, param));
+			param = (GdaHolder *) params->data;
+			attributes = gda_data_proxy_get_value_attributes (form->priv->proxy, row, i);
 			gnome_db_basic_form_entry_set_editable ((GnomeDbBasicForm *) form, 
 								param, !(attributes & GDA_VALUE_ATTR_NO_MODIF));
 			if (GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "_gnome_db_explicit_show")) == 1)
@@ -481,7 +483,6 @@ iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form)
 			if (!explicit_hide) 
 				gnome_db_basic_form_entry_show ((GnomeDbBasicForm *) form, 
 								param, !(attributes & GDA_VALUE_ATTR_UNUSED));
-			params = params->next;
 		}
 	}
 }
@@ -507,7 +508,7 @@ proxy_row_inserted_or_removed_cb (GdaDataProxy *proxy, gint row, GnomeDbRawForm
 {
 	if (gda_data_model_get_n_rows (GDA_DATA_MODEL (form->priv->proxy)) != 0)
 		if (gda_data_model_iter_get_row (form->priv->iter) == -1)
-			gda_data_model_iter_set_at_row (form->priv->iter, row > 0 ? row - 1 : 0);
+			gda_data_model_iter_move_at_row (form->priv->iter, row > 0 ? row - 1 : 0);
 }
 
 /*
@@ -523,13 +524,13 @@ gnome_db_raw_form_initialize (GnomeDbRawForm *form, GtkWidget *layout, GHashTabl
 	 */
 	if (layout && box_widgets) {
 		/* REM: the @box_widgets has keys for query fields, and we need to provide
-		   gnome_db_basic_form_new_in_layout() with a hash table which has keys for GdaParameterListNode;
+		   gnome_db_basic_form_new_in_layout() with a hash table which has keys for GdaSetNode;
 		   this is why we need to use another hash table: 'fbw' */
 
 		TO_IMPLEMENT;
 		/* GSList *list; */
-/* 		GdaParameter *param; */
-/* 		GdaParameterListNode *node; */
+/* 		GdaHolder *param; */
+/* 		GdaSetNode *node; */
 /* 		gpointer widget; */
 
 /* 		fbw = g_hash_table_new (NULL, NULL); */
@@ -551,19 +552,16 @@ gnome_db_raw_form_initialize (GnomeDbRawForm *form, GtkWidget *layout, GHashTabl
 	if (!layout && gda_data_proxy_is_read_only (form->priv->proxy))
 		gnome_db_basic_form_show_entry_actions (GNOME_DB_BASIC_FORM (form), FALSE);
 
-	list = GDA_PARAMETER_LIST (form->priv->iter)->parameters;
-	while (list) {
-		GdaParameterListNode *node;
+	for (list = GDA_SET (form->priv->iter)->holders; list; list = list->next) {
+		GdaSetNode *node;
 
-		node = gda_parameter_list_find_node_for_param (GDA_PARAMETER_LIST (form->priv->iter), 
-							       GDA_PARAMETER (list->data));
+		node = gda_set_get_node (GDA_SET (form->priv->iter), GDA_HOLDER (list->data));
 		g_assert (node);
-		if (node->hint & GDA_PARAMETER_LIST_PARAM_HIDE) {
+		if (node->hint &GDA_SET_HOLDER_HIDE) {
 			gnome_db_basic_form_entry_show (GNOME_DB_BASIC_FORM (form),
-							GDA_PARAMETER (list->data), FALSE);
+							GDA_HOLDER (list->data), FALSE);
 			g_object_set_data (G_OBJECT (list->data), "_gnome_db_explicit_show", GINT_TO_POINTER (1));
 		}
-		list = g_slist_next (list);
 	}
 
 	/* data display update */
@@ -583,7 +581,7 @@ action_new_cb (GtkAction *action, GnomeDbRawForm *form)
 	GSList *list;
 
 	if (form->priv->write_mode >= GNOME_DB_DATA_WIDGET_WRITE_ON_ROW_CHANGE) 
-		if (! gda_data_model_iter_can_be_moved (form->priv->iter))
+		if (! gda_set_is_valid (GDA_SET (form->priv->iter), NULL))
 			return;
 
 	/* append a row in the proxy */
@@ -595,19 +593,19 @@ action_new_cb (GtkAction *action, GnomeDbRawForm *form)
 		return;
 	}
 
-	g_assert (gda_data_model_iter_set_at_row (form->priv->iter, newrow));
+	g_assert (gda_data_model_iter_move_at_row (form->priv->iter, newrow));
 
 	/* set parameters to their default values */
-	list = GDA_PARAMETER_LIST (form->priv->iter)->parameters;
+	list = GDA_SET (form->priv->iter)->holders;
 	while (list) {
-		GdaParameter *param;
+		GdaHolder *param;
 		const GValue *value;
 		
 		g_object_get (G_OBJECT (list->data), "full_bind", &param, NULL);
 		if (! param) {
-			value = gda_parameter_list_get_param_default_value ((GdaParameterList *) (form->priv->iter),
-									    GDA_PARAMETER (list->data));
-			gda_parameter_set_value (GDA_PARAMETER (list->data), value);
+			value = gda_holder_get_default_value (GDA_HOLDER (list->data));
+			if (value)
+				gda_holder_set_value_to_default (GDA_HOLDER (list->data));
 		}
 		else
 			g_object_unref (param);
@@ -671,7 +669,7 @@ action_commit_cb (GtkAction *action, GnomeDbRawForm *form)
 					      actually succedded the commit 
 					      => we need to come back to that row
 					   */
-				gda_data_model_iter_set_at_row (form->priv->iter, row);
+				gda_data_model_iter_move_at_row (form->priv->iter, row);
 		}
 	}
 	else
@@ -907,7 +905,7 @@ arrow_actions_real_do (GnomeDbRawForm *form, gint movement)
 	}
 
 	if (oldrow != row)
-		gda_data_model_iter_set_at_row (form->priv->iter, row);
+		gda_data_model_iter_move_at_row (form->priv->iter, row);
 }
 
 /*
@@ -929,13 +927,13 @@ static void
 gnome_db_raw_form_col_set_show (GnomeDbDataWidget *iface, gint column, gboolean shown)
 {
 	GnomeDbRawForm *form;
-	GdaParameter *param;
+	GdaHolder *param;
 
 	g_return_if_fail (GNOME_DB_IS_RAW_FORM (iface));
 	form = GNOME_DB_RAW_FORM (iface);
 	g_return_if_fail (form->priv);
 
-	param = gda_data_model_iter_get_param_for_column (form->priv->iter, column);
+	param = gda_data_model_iter_get_holder_for_field (form->priv->iter, column);
 	g_return_if_fail (param);
 	gnome_db_basic_form_entry_show (GNOME_DB_BASIC_FORM (form), param, shown);
 	g_object_set_data (G_OBJECT (param), "_gnome_db_explicit_show", GINT_TO_POINTER (shown ? 2 : 1));
