@@ -1,6 +1,6 @@
 /* gnome-db-tools.h
  *
- * Copyright (C) 2005 - 2006 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2005 - 2008 Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,8 +28,8 @@
 G_BEGIN_DECLS
 
 void              gnome_db_util_init_plugins      ();
-GnomeDbDataEntry *gnome_db_util_new_data_entry    (GdaDict *dict, GType type, const gchar *plugin_name);
-GtkCellRenderer  *gnome_db_util_new_cell_renderer (GdaDict *dict, GType type, const gchar *plugin_name);
+GnomeDbDataEntry *gnome_db_util_new_data_entry    (GType type, const gchar *plugin_name);
+GtkCellRenderer  *gnome_db_util_new_cell_renderer (GType type, const gchar *plugin_name);
 
 G_END_DECLS
 
